''****************************************************************
''*  VGA 160x240 2-Color Bitmap Driver for TINY-VGA-System       *
''*  based on VGA 640x240 2-Color Bitmap Driver v1.0             *   
''*  (C) 2006, 2007 Parallax, Inc.                               *
''****************************************************************
''
'' This is Modification of the original VGA object arranged for use with the TINY-Computer
'' and my TINY-Emulator.
'' I needed a driver with a minumum of resolution and so a minimum of Screenbuffer
'' in a 2-Color- mode to drive my VGA-LCD-Monitor. 
'' Resolution is now 160x240 Pixel. It needs 300 Longs for Bitmapbuffer and
'' one Word for the colors.
'' A sync indicator signals each time the screen is drawn (you may ignore).
'' You must provide buffers for the colors, pixels, and sync. Once started,
'' all interfacing is done via memory. To this object, all buffers are read-
'' only, with the exception of the sync indicator which gets written with a
'' non-0 value. You may freely write all buffers to affect screen appearance.
'' Pixel memory is arranged left-to-right then top-to-bottom.      
'' Uwe Nickel
'' u.nickel@lycos.com

''

CON
' 160 x 240 
' 

  hp = 160      'horizontal pixels
  vp = 240      'vertical pixels
  hf = 8'12        'horizontal front porch pixels org 40
  hs = 40'48       'horizontal sync pixels
  hb = 13'88       'horizontal back porch pixels
  vf = 5        'vertical front porch lines - orig 9
  vs = 3        'vertical sync lines
  vb = 28       'vertical back porch lines - orig 28
  hn = 1        'horizontal normal sync state (0|1)
  vn = 1        'vertical normal sync state (0|1)
  pr = 20'12       'pixel rate in MHz at 80MHz system clock (5MHz granularity)

' Tiles

  xtiles = hp / 32
  ytiles = vp / 16 

' H/V inactive states
  
  hv_inactive = (hn << 1 + vn) * $0101

VAR 'long cog
    
PUB start(BasePin, ColorPtr, PixelPtr, SyncPtr) : okay 

'' Start VGA driver - starts a COG
'' returns false if no COG available
''
''     BasePin = VGA starting pin (0, 8, 16, 24, etc.)
''
''    ColorPtr = Pointer to Hubadresse (1 Word) which define the "0" and "1" colors for
''               all pixel. The lower byte of each word contains
''               the "0" bit RGB data while the upper byte of each word contains
''               the "1" bit RGB data for the associated group. The RGB
''               data in each byte is arranged as %RRGGBB00 (4 levels each).
''
''               color word example: %%0020_3300 = "0" = gold, "1" = blue
''
''    PixelPtr = Pointer to longs containing pixels that make up the (160 x
''               240) pixel bitmap. Longs' LSBs appear left on the screen, while
''               MSBs appear right. The longs are arranged in sequence from left-
''               to-right, then top-to-bottom.

''     SyncPtr = Pointer to long which gets written with non-0 upon each screen
''               refresh. May be used to time writes/scrolls, so that chopiness
''               can be avoided. You must clear it each time if you want to see
''               it re-trigger.

  'if driver is already running, stop it
 ' stop

  'implant pin settings and pointers, then launch COG
  reg_vcfg := $200000FF + (BasePin & %111000) << 6
  reg_dira := ($FF << (BasePin & %011000)) & (BasePin & %100000 == 0) 'i & j
  
  color_adr := ColorPtr
  pixel_base := PixelPtr
 
'  if (cog := cognew(@init, SyncPtr) + 1)
'    return true
  cognew(@init, SyncPtr)

{PUB stop | i

'' Stop VGA driver - frees a COG

  if cog
    cogstop(cog~ - 1)
}

DAT

'***********************************************
'* Assembly language VGA 2-color bitmap driver *
'***********************************************

                        org  0                          'set origin to $000 for start of program

' Initialization code - init I/O
                                                                                              
init                    mov     dira,reg_dira           'set pin directions                   
                 
                        movi    ctra,#%00001_100        'enable PLL in ctra (VCO runs at 4x)
                  '      movi    frqa,#(pr / 5) << 3     'set pixel rate for 640 and 320                                    
                        movi    frqa,#(pr / 6) << 3     'set pixel rate for 160    
                        mov     vcfg,reg_vcfg           'set video configuration
                        
' Main loop, display field and do invisible sync lines
                          
field                   mov     pixel_ptr,pixel_base    'reset pixel pointer

                        mov     yl_set, #4'8            'value mode 0
                        mov     yx_set, #8'4 
:md                     mov     y,#ytiles               'set y tiles
:ytile                  mov     yl,yl_set               'set y lines per tile
:yline                  mov     yx,yx_set               'set y expansion                        
:yexpand                mov     x,#xtiles               'set x tiles
                        mov     vscl,vscl_pixel         'set pixel vscl

:xtile                  rdword  color,color_adr 'base 'ptr         'get color word 
                        and     color,colormask         'clear h/v bits
                        or      color,hv                'set h/v inactive states             
                        rdlong  pixel,pixel_ptr         'get pixel long
                        waitvid color,pixel             'pass colors and pixels to video
                        add     pixel_ptr,#4            'point to next pixel long
                        djnz    x,#:xtile               'another x tile?
                        sub     pixel_ptr,#xtiles * 4   'repoint to first pixels in same line

                        mov     x,#1                    'do horizontal sync
                        call    #hsync

                        djnz    yx,#:yexpand            'y expand - gleiche Zeile wiederholen
                        
                        add     pixel_ptr,#xtiles * 4   'point to first pixels in next line
                        djnz    yl,#:yline              'another y line in same tile?
                        djnz    y,#:ytile               'another y tile?


                        wrlong  colormask,par           'visible done, write non-0 to sync
                        
                        mov     x,#vf                   'do vertical front porch lines
                        call    #blank
                        mov     x,#vs                   'do vertical sync lines
                        call    #vsync
                        mov     x,#vb                   'do vertical back porch lines
                        call    #vsync

                        jmp     #field                  'field done, loop
                        

' Subroutine - do blank lines

vsync                   xor     hvsync,#$101            'flip vertical sync bits

blank                   mov     vscl,hvis               'do blank pixels
                        waitvid hvsync,#0
hsync                   mov     vscl,#hf                'do horizontal front porch pixels
                        waitvid hvsync,#0
                        mov     vscl,#hs                'do horizontal sync pixels
                        waitvid hvsync,#1
                        mov     vscl,#hb                'do horizontal back porch pixels
                        waitvid hvsync,#0
                        djnz    x,#blank                'another line?
hsync_ret
blank_ret
vsync_ret               ret


' Data
reg_dira                long    0                       'set at runtime
reg_vcfg                long    0                       'set at runtime
color_adr               long    0                       'set at runtime (2 contiguous longs)
pixel_base              long    0                       'set at runtime
vscl_pixel              long    1 << 12 + 32            '1 pixel per clock and 32 pixels per set
colormask               long    $FCFC                   'mask to isolate R,G,B bits from H,V
hvis                    long    hp                      'visible pixels per scan line
hv                      long    hv_inactive             '-H,-V states
hvsync                  long    hv_inactive ^ $200      '+/-H,-V states

' Uninitialized data
pixel_ptr               res     1
color                   res     1
pixel                   res     1
x                       res     1
y                       res     1
yl                      res     1
yx                      res     1
yl_set                  res     1
yx_set                  res     1

fit $1f0